import java.util.Vector;

/**
 * abstract class for communicating with others
 *
 *
 *
 */
abstract public class AbstractMessageHandler {

    /* Storage for registered change listeners. */
    protected transient Vector listenerList = new Vector();

    /**
     * Registers an object to receive notification
     * @param listener the object to register.
     */
    public void addMessageListener(MessageListener listener) {
        this.listenerList.addElement(listener);
    }

    /**
     * Deregisters an object so that it no longer receives notification
     * @param listener the object to deregister.
     */
    public void removeMessageListener(MessageListener listener) {
        this.listenerList.removeElement(listener);
    }

    /**
     * Let all listeners know when a message has been received
     * @param opcode, the received opcode
     * @param distance, the received distance
     */
    abstract protected void fireMessageReceived(Packet packet);
}
