
import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * class used for communicating with the
 * controller over a network-connection
 *
 * @author Cengiz Genc,Stephan Poser
 */
public class BrainMessageHandler extends AbstractMessageHandler
{

    // The port we use to communicate
    protected ServerSocket fromBrain = null;
    protected Socket toBrain = null;
    
    // The brain input/output streams
    protected DataInputStream brainOutput = null;
    protected DataOutputStream brainInput = null;
    private boolean connected;
    /**
     * Create the default message handler
     */
    public BrainMessageHandler(int port)
    {
    	try
	{
            fromBrain = new ServerSocket(port);
            // Create the subthread to receive messages
            ReceiveMessageThread thread = new ReceiveMessageThread();
            connected=false;
	}
	catch(Exception e)
        {
            System.out.println(e.toString());
	}
    }

    /** Logs the client in and creates the streams to write to and read from
     *
     * @return
     */
    public void checkin()
    {
            try
            {
                    toBrain= fromBrain.accept();
                    if(toBrain!=null)
                    {
                            brainOutput = new DataInputStream(toBrain.getInputStream());
                            brainInput = new DataOutputStream(toBrain.getOutputStream());
                            connected=true;
                    }
            }
            catch(Exception e)
            {

                System.out.println(e.toString());
            }
    }

    public boolean isActive()
    {
        // if a client is connected to the Server the method returns true,otherwise it gets a timeout(5s)
        try{
            InputStream test = toBrain.getInputStream();
            return test!=null & connected==true;
        }
        catch(Exception e)
        {
            return false;
        }
    }

    /**
     * Send a specific Message to the Brain
     * @param packet, the Packet to send
     */
    public void sendMessageToBrain(Packet packet)
    {
            try {
            	// Send the opcode
            	this.brainInput.writeInt(packet.getOpcode());
            	this.brainInput.writeInt(packet.getData());
            } 
            catch (IOException e) 
            {
            	System.err.println("ProxyMessage: sendMessageToRCX(Packet): " + e);
            }
    }


    /**
     * Let all listeners know when a message has been received
     * @param opcode, the received opcode
     * @param distance, the received distance
     */
    protected void fireMessageReceived(Packet packet)
    {
            // Go through the whole listener list
            for(int i = 0; i < listenerList.size(); i++)
                    ((MessageListener)(listenerList.elementAt(i))).readFromSource(packet);
    }


	class ReceiveMessageThread extends Thread {

		// The opcode we received
		protected Packet packet;
		protected int opcode = -1;
		protected int value = -1;

		// The number of bytes we need to read
		protected int numberOfBytes = 0;
		/**
		 * Construct a new ReceiveMessageThread
		 */
		public ReceiveMessageThread() {
			super("ReceiveMessage");

			// Start this thread
			this.start();
		}

		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run() {

			// Run forever
			while(true)
			{
				try {
					//Waiting for connection,if no connection is established
					if (isActive() == false)
					{
						checkin();
					}

					// Read the opcode
					opcode = brainOutput.readInt();
		
					// Read the value
					value = brainOutput.readInt();

					// We got the full message, let all listeners know
					fireMessageReceived(new Packet(opcode, value));
				}
				//client has been disconnected:
				catch (IOException e)
				{
					connected = false;
				}
				catch(NullPointerException e)
				{}
			}
		}

		/**
		 * Check the opcode and return the number of additional bytes we need
		 * @param opcode, the opcode to check
		 * @return the number of additional bytes we need for this message
		 */
		private int checkOpcode(int opcode)
		{
			//OPCODE_ACTIVE is the only 1byte-opcode:
			if(opcode == IRProtocol.OPCODE_ACTIVE)
				return 0;
			else
				return 1;
		}
	}

}
