/**
 *                      Remotely Controlled Robot
 *                    FHM Munich FB07 Computer Science
 *                         WS2004/05 Group 1
 *                          Proxy Module
 *
 *
 * Class Description:
 *          This class takes care of sending the packets, received
 *          from the controlsoftware, to the rcx and vice versa
 *          This is the main-class which should be called from
 *          commandline
 *
 * System:
 *          Linux 2.6, Sun Java 1.4.2; Win32, Sun Java 1.4.2
 *
 * @author Cengiz Genc <c.genc@gmx.de>
 * @author Stephan Poser <stephan.poser@gmx.de>
 *
 *
 * $Id: rcxproxy.java 165 2004-12-09 18:51:31Z poser $
 *
 */

import java.net.ServerSocket;
import java.net.Socket;
import java.io.InputStream;
import java.io.OutputStream;

public class rcxproxy implements proxy, MessageListener{

    public rcxproxy()
    {

    }
    /**
     * Creates a new instance of rcxproxy
     * @param port
     */
    public rcxproxy(BrainMessageHandler brainMessageHandler, RcxMessageHandler rcxMessageHandler)
    {
		//proxy <-> brain handler
		this.brainMessageHandler = brainMessageHandler;
		this.brainMessageHandler.addMessageListener(this);

		//proxy <-> rcx handler
		this.rcxMessageHandler = rcxMessageHandler;
		this.rcxMessageHandler.addMessageListener(this);

    }

    public boolean isActive()
    {
		return brainMessageHandler.isActive();
    }



    /** I/O method for the rcx->proxy exchange
     * called when packet received
     * @return
     */
    public void readFromDest(Packet packet)
    {
		System.out.println("Received from RCX: " + packet);

		//received packet from rcx -> send it to brain
		writeToSource(packet);
    }

    /** I/O method for the proxy->rcx exchange
     * @param packet : packet that has to be sent to the rcx
     * @return
     */
    public void writeToDest(Packet packet)
    {
		System.out.println("Sending to RCX: " + packet);

		try {
			//send packet to rcx
			rcxMessageHandler.sendMessageToRCX(packet);
		}
		catch(LostConnectionException ex)
		{
			System.err.println("\n[Error] connection to Robot lost, trying to resend last packet in 10sec\n");
			try {
				Thread.sleep(10000);
			}
			catch(InterruptedException exc) {}

			writeToDest(packet);
		}
		catch(IllegalArgumentException ex)
		{/*tried to send damaged packets or value out of range ... ignore this packet*/}
    }


    /** I/O for controlsoftware -> proxy exchange
     * called when packet received
     * @return
     */
    public void readFromSource(Packet packet)
    {
	//received packet from brain -> send it to rcx
        System.out.println("Received from Brain: "+packet);
	writeToDest(packet);
    }

    /**I/O for proxy->controlsoftware exchange
     * @param packet: information that has to be sent to the control software
     * @return
     */
    public void writeToSource(Packet packet)
    {
        System.out.println("Sending to Brain: "+packet);
        brainMessageHandler.sendMessageToBrain(packet);
    }


    public static void main(String[] args)
    {
		// Create the communication handler
		RcxMessageHandler rcxMessageHandler = new RcxMessageHandler();
		BrainMessageHandler brainMessageHandler = new BrainMessageHandler(1980);

		rcxproxy p = new rcxproxy(brainMessageHandler, rcxMessageHandler);
		System.out.println("Proxy - group 1");
		System.out.println("Waiting for connection....");

		while( !p.isActive() )
		{
			//do nothint and wait
		}
		System.out.println("Client connected!\n\n");

    }

    /* the communication handler */
    protected RcxMessageHandler rcxMessageHandler = null;
    /* the communication handler */
    protected BrainMessageHandler brainMessageHandler = null;

}
